//
//  ViewController.m
//  MVisaSDKAppObjC
//
//

#import "ViewController.h"
#import "MVisaSDK/MVisaSDK.h"
#import "ViewController+Merchant.h"

@interface ViewController () <MVisaPayMerchantDelegate, MVisaPayFriendDelegate, MVisaWithdrawCashDelegate, UITableViewDataSource, UITableViewDelegate>
@property (weak, nonatomic) IBOutlet UITableView *tableView;
@end


@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    _tableView.backgroundColor = [UIColor colorWithRed:0.08 green:0.10 blue:0.32 alpha:1.0];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - TableView Delegates
-(NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 2;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    
    if (section == 0) {
        return 4;
    }
    return 2;
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section {
    return 28;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 70;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    
    if (section == 0) {
        return @"Consumer Flow";
    }
    
    return @"Merchant Flow";
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *tableCellId = @"cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier: tableCellId];
    if(cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:tableCellId];
    }
    cell.backgroundColor = [UIColor colorWithRed:0.08 green:0.10 blue:0.32 alpha:1.0];
    cell.textLabel.textColor = [UIColor whiteColor];
    
    if (indexPath.section == 0) {
        if (indexPath.row == 0) {
            cell.textLabel.text = @"Pay Merchant";
        } else if (indexPath.row == 1) {
            cell.textLabel.text = @"Pay a Friend";
        } else if (indexPath.row == 2) {
            cell.textLabel.text = @"Withdraw Cash";
        } else if (indexPath.row == 3) {
            cell.textLabel.text = @"Pay Merchant Alias";
        }
    }
    else {
        
        if(indexPath.row == 0) {
            cell.textLabel.text = @"Request Payment";
        } else if(indexPath.row == 1) {
            cell.textLabel.text = @"Refund";
        }
    }
    
    return cell;
}

-(void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    if (indexPath.section == 0) {
        if (indexPath.row == 0) {
            [self initiatePayMerchantFlow];
        } else if (indexPath.row == 1) {
            [self initiatePayFriendFlow];
        } else if (indexPath.row == 2) {
            [self initiateWithdrawCashFlow];
        } else if (indexPath.row == 3) {
            [self initiatePayMerchantAliasFlow];
        }
    }
    else {
        if(indexPath.row == 0) {
            [self initiateRequestPaymentFlow];
        } else if(indexPath.row == 1) {
            [self initiateRefundFlow];
        }
    }
}


#pragma mark - MVisaSDK Integration Methods

// code to launch pay friend flow
- (void)initiatePayMerchantFlow {
    // create and set one time config
    MVisaConfig *config = [[MVisaConfig alloc]init];
    config.backgroundColor = UIColor.redColor;
    config.actionButtonActiveColor = UIColor.blueColor;
    config.actionButtonColor = UIColor.orangeColor;
    [MVisaSDK setupOneTimeConfiguration:config];
    
    // create consumer card list to be used to create pay merchant request
    MVisaCardDetails *card1 = [[MVisaCardDetails alloc] initWithLastFourDigits:@"5678" cardType: @"Visa Classic" issuerLogo: nil cardArtColor: nil cardArtOverlay: nil networkType: MVisaCardTypeMastercard];
    MVisaCardDetails *card2 = [[MVisaCardDetails alloc] initWithLastFourDigits:@"1357" cardType: @"Visa Platinum" issuerLogo: nil cardArtColor: nil cardArtOverlay: nil networkType: MVisaCardTypeRupay];
    MVisaCardDetails *card3 = [[MVisaCardDetails alloc] initWithLastFourDigits:@"1257" cardType: @"Visa Platinum" issuerLogo: nil cardArtColor: nil cardArtOverlay: nil networkType: MVisaCardTypeVisa];
    NSArray *cards = @[card1, card2, card3];
    
    // create pay merchant request
    /** Deprecated in 2.4.0 use this initializer method for old SDK version 2.3.0 */
    //MVisaPayMerchantRequest *request = [[MVisaPayMerchantRequest alloc] initWithCards: cards currencyCode: @"840" defaultCardIndex: 0 qrTlvStr:nil];
    
    //initializer method for version 2.4.0
    MVisaPayMerchantRequest *request = [[MVisaPayMerchantRequest alloc]initWithCards:cards currencyCode:@"840" defaultCardIndex:0 qrTlvStr:nil shouldShowCurrencyDropdown:true];
    

    // launch the MVisa pay merchant flow
    // overload function can also be used with a completion block
    [MVisaSDK launchPayMerchantFlowWithRequest:request delegate:self];
    
}

// delegate method called on completion of pay merchant flow
- (void)payMerchantFlowDidFinish:(BOOL)successful withResponse:(MVisaPayMerchantResponse * _Nullable)payMerchantResponse withError:(MVisaError * _Nullable)error {
    
    if(successful) {
        NSLog(@"mVisaMerchantPan : %@",payMerchantResponse.mVisaMerchantPAN);
        NSLog(@"merchantName : %@",payMerchantResponse.merchantName);
        ///likewise you print other veriable from MVisaPayMerchantResponse
    } else {
        NSLog(@"MVisa pay merchant flow was unsuccessful");
    }
    // close the MVisaSDK UI and return to host app
    [MVisaSDK endMVisaFlow:nil];
    
}

// code to launch pay friend flow
- (void)initiatePayMerchantAliasFlow {
    
    // create and set one time config
    MVisaConfig *config = [[MVisaConfig alloc]init];
    [MVisaSDK setupOneTimeConfiguration:config];
    
    // create consumer card list to be used to create pay merchant request
    MVisaCardDetails *card1 = [[MVisaCardDetails alloc] initWithLastFourDigits:@"5678" cardType: @"Visa Classic" issuerLogo: nil cardArtColor: nil cardArtOverlay: nil networkType: MVisaCardTypeMastercard];
    MVisaCardDetails *card2 = [[MVisaCardDetails alloc] initWithLastFourDigits:@"1357" cardType: @"Visa Platinum" issuerLogo: nil cardArtColor: nil cardArtOverlay: nil networkType: MVisaCardTypeRupay];
    MVisaCardDetails *card3 = [[MVisaCardDetails alloc] initWithLastFourDigits:@"1257" cardType: @"Visa Platinum" issuerLogo: nil cardArtColor: nil cardArtOverlay: nil networkType: MVisaCardTypeVisa];
    NSArray *cards = @[card1, card2, card3];
    
    // create pay merchant request
    MVisaPayMerchantRequest *request = [[MVisaPayMerchantRequest alloc] initWithCards: cards currencyCode: @"840" defaultCardIndex: 0 qrTlvStr:nil];
    
    // launch the MVisa pay merchant flow
    // overload function can also be used with a completion block
    [MVisaSDK launchPayMerchantAliasFlowWithRequest:request aliasValidationHandler:^(PayMerchantAliasData * _Nullable aliasData, PayMerchantAliasResponse * _Nullable aliasResponse) {
        
        AliasResponse *response = [[AliasResponse alloc] initWithMerchantName:@"Corner Store"
                                                                   merchantId:nil
                                                                    payeeCity:nil
                                                             payeeCountryCode:nil];
        
        AliasError *error = [[AliasError alloc] initWithError:@"Alias not found for the given number"];
        NSLog(@"%@", error);
        
        [aliasResponse setDataWithAliasResponse:response error:nil];
        
    } completionHandler:^(BOOL successful, MVisaPayMerchantResponse * _Nullable payMerchantResponse, MVisaError * _Nullable error) {
        
        if(successful) {
            NSLog(@"mVisaMerchantPan : %@",payMerchantResponse.mVisaMerchantPAN);
            NSLog(@"merchantName : %@",payMerchantResponse.merchantName);
            ///likewise you print other veriable from MVisaPayMerchantResponse
        } else {
            NSLog(@"MVisa pay merchant flow was unsuccessful");
        }
        // close the MVisaSDK UI and return to host app
        [MVisaSDK endMVisaFlow:nil];
    }];
}

// code to launch pay friend flow
- (void)initiatePayFriendFlow {
    
    // create and set one time config
    MVisaConfig *config = [[MVisaConfig alloc]init];
    [MVisaSDK setupOneTimeConfiguration:config];
    
    // create payee list to be used to create pay friend request
    MVisaPayee *payee1 = [[MVisaPayee alloc] initWithPayeeName:@"John Smith" lastFour:@"1234" mobileNumber:nil bankName:nil];
    MVisaPayee *payee2 = [[MVisaPayee alloc] initWithPayeeName:@"Harry Potter" lastFour:@"5680" mobileNumber:nil bankName:nil];
    MVisaPayee *payee3 = [[MVisaPayee alloc] initWithPayeeName:@"Sarah Blackmon" lastFour:@"5739" mobileNumber:nil bankName:nil];
    NSArray *payees = @[payee1, payee2, payee3];
    
    // create consumer card list to be used to create pay friend request
    MVisaCardDetails *card1 = [[MVisaCardDetails alloc] initWithLastFourDigits:@"1234" cardType:@"Visa Classic" issuerLogo:nil cardArtColor:nil cardArtOverlay:nil networkType:MVisaCardTypeVisa];
    MVisaCardDetails *card2 = [[MVisaCardDetails alloc] initWithLastFourDigits:@"5678" cardType:@"Visa Platinum" issuerLogo:nil cardArtColor:nil cardArtOverlay:nil networkType:MVisaCardTypeRupay];
    NSArray *cards = @[card1, card2];
    
    // create pay merchant request
    MVisaPayFriendRequest *request = [[MVisaPayFriendRequest alloc] initWithCards:cards currencyCode:@"840" payees:payees defaultCardIndex:0];
    
    // launch the MVisa pay friend flow
    // overload function can also be used with a completion block
    [MVisaSDK launchPayFriendFlowWithRequest:request delegate:self];
    
}

- (void)payFriendFlowDidFinish:(BOOL)successful withResponse:(MVisaPayFriendResponse *)payFriendResponse withError:(MVisaError *)error {
    
    if(successful) {
        NSLog(@"payeeIndex: %ld", (long)payFriendResponse.payeeIndex);
        NSLog(@"print other response fields here");
    } else {
        NSLog(@"MVisa pay friend flow was unsuccessfull");
    }
    
    // close the MVisaSDK UI and return to host app
    [MVisaSDK endMVisaFlow:nil];
}

// code to launch withdraw cash flow
- (void)initiateWithdrawCashFlow {
    // create and set one time config
    MVisaConfig *config = [[MVisaConfig alloc]init];
    [MVisaSDK setupOneTimeConfiguration:config];
    
    // create consumer card list to be used to create withdraw cash request
    MVisaCardDetails *card1 = [[MVisaCardDetails alloc] initWithLastFourDigits:@"5678" cardType: @"Visa Classic" issuerLogo: nil cardArtColor: nil cardArtOverlay: nil networkType: MVisaCardTypeMastercard];
    MVisaCardDetails *card2 = [[MVisaCardDetails alloc] initWithLastFourDigits:@"1357" cardType: @"Visa Platinum" issuerLogo: nil cardArtColor: nil cardArtOverlay: nil networkType: MVisaCardTypeRupay];
    MVisaCardDetails *card3 = [[MVisaCardDetails alloc] initWithLastFourDigits:@"1257" cardType: @"Visa Platinum" issuerLogo: nil cardArtColor: nil cardArtOverlay: nil networkType: MVisaCardTypeVisa];
    NSArray *cards = @[card1, card2, card3];
    
    // create withdraw cash request
    MVisaWithdrawCashRequest *request = [[MVisaWithdrawCashRequest alloc] initWithCards: cards currencyCode: @"840" defaultCardIndex: 0 serviceFee: @"200.00"];
    
    // launch the MVisa withdraw cash flow
    // overload function can also be used with a completion block
    [MVisaSDK launchWithdrawCashFlowWithRequest: request delegate: self];
    
}

// delegate method called on completion of withdraw cash flow
- (void)withdrawCashFlowDidFinish:(BOOL)successful withResponse:(MVisaWithdrawCashResponse * _Nullable)withdrawCashResponse withError:(MVisaError * _Nullable)error {
    
    if(successful) {
        NSLog(@"agentID : %@",withdrawCashResponse.agentID);
        NSLog(@"agentName : %@",withdrawCashResponse.agentName);
        ///likewise you print other veriable from MVisaWithdrawCashResponse
    } else {
        NSLog(@"MVisa withdraw cash flow was unsuccessful");
    }
    // close the MVisaSDK UI and return to host app
    [MVisaSDK endMVisaFlow:nil];
}

@end
